<?php
/**
 * @file
 * This file contains helper functions for openlucius.
 */

/**
 * Function to obtain an icon based on type / path.
 *
 * @param string $type
 *   Either the node type or the path.
 *
 * @return string
 *   Returns a usable glyphicon, defaults to globe.
 */
function openlucius_core_icons($type) {

  $icons = array(
    'ol_group'             => 'record',
    'file'                 => 'file',
    'ol_message'           => 'envelope',
    'ol_text_document'     => 'font',
    'ol_todo_list'         => 'list-alt',
    'ol_todo'              => 'inbox',
    'ol_event'             => 'calendar',
    'group-files'          => 'file',
    'all-files'            => 'file',
    'group-messages'       => 'envelope',
    'all-messages'         => 'envelope',
    'group-textdocuments'  => 'font',
    'all-text-documents'   => 'font',
    'group-todo-lists'     => 'list-alt',
    'all-todo-lists'       => 'list-alt',
    'group-calendar/month' => 'calendar',
    'all-calendar'         => 'calendar',
    'trash'                => 'trash',
    'archived-groups'      => 'folder-close',
    'comment'              => 'comment',
    'group-users'          => 'user',
    'all-users'            => 'user',
    'companies-teams'      => 'user',
    'blocked-users'        => 'user',
    'user'                 => 'user',
    'user-calendar'        => 'inbox',
    'search'               => 'search',
    'node_add'             => 'plus-sign',
    'node_due_date'        => 'time',
    'node_label'           => 'flag',
    'node_time'            => 'time',
    'node_show_clients'    => 'eye-open',
    'node_hide_clients'    => 'eye-close',
  );

  // Allow other modules to add / edit icons.
  drupal_alter('openlucius_core_icons', $type);

  return isset($icons[$type]) ? $icons[$type] : 'globe';
}

/**
 * Function to fetch all user ids.
 * @return array
 *   Returns an array containing all of the active user id's.
 */
function openlucius_core_fetch_all_active_uids() {
  return db_select('users', 'u')
    ->fields('u', array('uid'))
    ->condition('status', 1, '=')
    ->execute()
    ->fetchCol();
}

/**
 * Function for quickly loading a user image in a given style.
 *
 * @param mixed $u
 *   Either a user id or a fully loaded user object.
 * @param string $style
 *   An image style to be applied to the image.
 *
 * @return string
 *   Returns the path to the styled image.
 * @throws \Exception
 */
function openlucius_core_fetch_user_image($u, $style = 'ol_50x50') {

  // If u is a number it's the uid otherwise it is a user object.
  $account = is_numeric($u) ? user_load($u) : $u;

  // The default 'avatar' image.
  $default_uri = drupal_get_path('profile', 'openlucius') . '/themes/openlucius/images/avatar.png';

  // Check if the user has a picture.
  if (!empty($account->picture)) {

    // Check if the file was loaded.
    if (!empty($account->picture->uri)) {

      if (file_exists($account->picture->uri)) {
        $image = theme('image_style', array(
          'style_name' => $style,
          'path'       => $account->picture->uri,
        ));
      }
      else {
        $image = theme('image', array(
          'style_name' => 'ol_50x50',
          'path'       => $default_uri,
        ));
      }
    }
    // Build path to default picture.
    else {
      $image = theme('image', array(
        'style_name' => 'ol_50x50',
        'path'       => $default_uri,
      ));
    }
  }

  // Build path to default picture.
  else {
    $image = theme('image', array(
      'style_name' => 'ol_50x50',
      'path'       => $default_uri,
    ));
  }

  return $image;
}

/**
 * Function to fetch the read nodes based on user id and an array of nodes.
 *
 * @param int $uid
 *   The user for whom the history tables should be read.
 * @param array $node_ids
 *   The node id's to be checked.
 * @param bool $data
 *   Whether the node-data should be returned or the count.
 *
 * @return mixed
 *   Returns either FALSE when not found or an array containing the node id's
 *   or the amount of items.
 */
function openlucius_core_fetch_read_nodes($uid, array $node_ids, $data = FALSE) {

  // Fetch read nodes based on node ids.
  $query = db_select('history', 'h')
    ->fields('h', array('nid'))
    ->condition('uid', $uid, '=')
    ->condition('nid', $node_ids, 'IN')
    ->execute();

  // Return row count.
  if (!$data) {
    return $query->rowCount();
  }

  // Return node id's.
  return $query->fetchCol();
}

/**
 * Function for fetching all published nodes id's of a certain type.
 *
 * @param string $type
 *   The node type to searched.
 *
 * @return mixed
 *   Returns either FALSE when not found or an array containing the node id's.
 */
function openlucius_core_fetch_nodes_of_type($type) {

  // All nodes of type.
  return db_select('node', 'n')
    ->fields('n', array('nid'))
    ->condition('n.type', check_plain($type), '=')
    ->condition('n.status', 1, '=')
    ->execute()
    ->fetchCol();
}

/**
 * Function to render a node field for the metadata block.
 *
 * @param \stdClass $node
 *   The node containing the data.
 *
 * @return string
 *   The rendered field.
 */
function openlucius_core_node_render_meta_fields(\stdClass $node) {
  $fields = array(
    'field_todo_list_reference'      => openlucius_core_icons('ol_todo_list'),
    'field_todo_user_reference'      => openlucius_core_icons('user'),
    'field_todo_due_date_singledate' => openlucius_core_icons('node_due_date'),
    'field_todo_label'               => openlucius_core_icons('node_label'),
    'field_shared_show_clients'      => openlucius_core_icons('node_show_clients'),
  );

  // Allow other modules to alter the fields / icons.
  drupal_alter('openlucius_core_node_meta_fields', $fields);

  // Initialize empty array for field renders.
  $list = array();

  // Load the wrapper.
  $node_wrapper = entity_metadata_wrapper('node', $node);

  // Loop through the fields.
  foreach ($fields as $field => $icon) {

    // Skip if the user is a client.
    if ($field == 'field_shared_show_clients' && openlucius_core_user_is_client()) {
      continue;
    }
    // Skip if the node is a text document as well.
    if ($field == 'field_shared_show_clients' && $node->type == 'ol_text_document') {
      continue;
    }

    // Check if we have the field.
    if (isset($node->{$field})) {

      // For all field which are not a label.
      if ($field !== 'field_todo_label') {

        // Fetch field items.
        $items      = field_get_items('node', $node, $field);
        $field_data = field_view_value('node', $node, $field, $items[0]);
        $render     = render($field_data);
      }

      // The todo label should only contain the name.
      else {
        $data   = $node_wrapper->{$field}->value();
        $render = $data->name;
      }

      if (!empty($render)) {
        // Append items to list.
        $list[] = '<i class="glyphicon glyphicon-' . $icon . '"></i>&nbsp;' . $render;
      }
    }
  }

  return theme('item_list', array('items'      => $list,
                                  'attributes' => array('class' => array('openlucius_meta_info'))
  ));
}

/**
 * Function for loading a user by realname.
 *
 * @param string $name
 *   The realname for this user.
 *
 * @return bool|mixed
 *   Returns either the loaded user or FALSE when not found.
 */
function openlucius_core_realname_load_user_by_name($name) {
  $uid = db_select('realname', 'r')
    ->fields('r', array('uid'))
    ->condition('r.realname', $name, '=')
    ->execute()
    ->fetchField();

  // Check if the user was found.
  if (!empty($uid)) {
    return user_load($uid);
  }

  return FALSE;
}
